#include "AppADC.h"
#include "asf.h"

// Kanl ADC, na ktorom je pripojen potenciometer
#define ADC_CHANNEL_POTENTIOMETER ADC_CHANNEL_1
// Referenn naptie (max. naptie) na senzore teploty
#define VOLT_REF (3300)
// Max. digitlna hodnota ADC
#define MAX_DIGITAL (4095)

// Posledn nameran hodnota potenciometra
static volatile uint32_t __appadc_last_potentiometer = 0;
// Posledn nameran hodnota senzora teploty
static volatile uint32_t __appadc_last_temperature = 0;
// Uruje, i je ADC povolen
static volatile bool __appadc_enabled = false;

// Inicializuje ADC pre pouitie v USB aplikcii
void usbapp_adc_initialize(void)
{
	// Povoli ADC obvod
	pmc_enable_periph_clk(ID_ADC);
	
	// Inicializova ADC
	adc_init(ADC, sysclk_get_cpu_hz(), 6400000, 8);
	adc_configure_timing(ADC, 0, ADC_SETTLING_TIME_3, 1);
	
	// Vyvola konverziu kdom
	adc_configure_trigger(ADC, ADC_TRIG_SW, 0);
	adc_check(ADC, sysclk_get_cpu_hz());
	
	// Povoli vstupn kanl pre senzor teploty
	adc_enable_channel(ADC, ADC_TEMPERATURE_SENSOR);
	
	// Povoli vstupn kanl pre potenciometer
	adc_enable_channel(ADC, ADC_CHANNEL_POTENTIOMETER);
	
	// Povoli senzor teploty
	adc_enable_ts(ADC);
	
	__appadc_enabled = true;

	// Povoli interupty pre ADC
	NVIC_EnableIRQ(ADC_IRQn);
}

// Zska hodnotu potenciometra
uint32_t usbapp_adc_read_potentiometer(void)
{
	if (__appadc_enabled)
		__appadc_last_potentiometer = adc_get_channel_value(ADC, ADC_CHANNEL_POTENTIOMETER);
	return __appadc_last_potentiometer;
}

// Spust konverziu ADC
void usbapp_adc_convert(void)
{
	if (__appadc_enabled)
		adc_start(ADC);
}

// Zska teplotu
float usbapp_adc_read_temperature(uint16_t * temperatureDigital)
{
	static uint32_t tsValue;
	
	if (__appadc_enabled)
		__appadc_last_temperature = adc_get_channel_value(ADC, ADC_TEMPERATURE_SENSOR);
	tsValue = __appadc_last_temperature;
	*temperatureDigital = tsValue;
	tsValue = tsValue * VOLT_REF / MAX_DIGITAL;
	return ((float)(tsValue - 800) * 0.37736 + 27.0);
}

// Povol vstupy pre ADC
void usbapp_adc_enable_channels(void)
{
	// Povoli vstupn kanl pre senzor teploty
	adc_enable_channel(ADC, ADC_TEMPERATURE_SENSOR);
	
	// Povoli vstupn kanl pre potenciometer
	adc_enable_channel(ADC, ADC_CHANNEL_POTENTIOMETER);
	
	// Povoli senzor teploty
	adc_enable_ts(ADC);
}

// Zru vstupy pre ADC
void usbapp_adc_disable_channels(void)
{
	adc_disable_all_channel(ADC);
	
	// Povoli senzor teploty
	adc_disable_ts(ADC);
}