#include "AppDisplay.h"
#include "asf.h"

// Max. poet riadkov na vku LCD
#define MAX_LINES 20
// Vka jednho riadku v pixeloch
#define LINE_HEIGHT 16

// truktra pre konfigurciu LCD
struct hx8347a_opt_t g_hx8347a_display_opt;
// Poet zobrazench riadkov
static volatile uint8_t __applcd_num_lines = 0;
// Hodnota urujca, i je LCD zapnut
static volatile bool __applcd_display_on = false;
// Hodnota urujca, i je podsvietenie zapnut
static volatile bool __applcd_backlight_on = false;
// rove podsvietenia
static volatile uint8_t __applcd_backlight = 0;
// Skrolovanie displeja
static volatile uint16_t __applcd_scroll = 0;

// Skonvertuje 24-bitov farbu na 16-bitov
hx8347a_color_t rgb24_to_rgb16(uint32_t ul_color)
{
	hx8347a_color_t result_color;
	result_color = (((ul_color >> 8) & 0xF800) |
	((ul_color >> 5) & 0x7E0) | ((ul_color >> 3) & 0x1F));
	return result_color;
}

// Inicializuje displej
void usbapp_lcd_initialize(void)
{
	// Povoli obvod SMC (Static Memory Controller)
	pmc_enable_periph_clk(ID_SMC);

	smc_set_setup_timing(SMC, CONF_BOARD_HX8347A_LCD_CS, SMC_SETUP_NWE_SETUP(1)
	| SMC_SETUP_NCS_WR_SETUP(1)
	| SMC_SETUP_NRD_SETUP(9)
	| SMC_SETUP_NCS_RD_SETUP(9));
	smc_set_pulse_timing(SMC, CONF_BOARD_HX8347A_LCD_CS, SMC_PULSE_NWE_PULSE(4)
	| SMC_PULSE_NCS_WR_PULSE(4)
	| SMC_PULSE_NRD_PULSE(36)
	| SMC_PULSE_NCS_RD_PULSE(36));
	smc_set_cycle_timing(SMC, CONF_BOARD_HX8347A_LCD_CS, SMC_CYCLE_NWE_CYCLE(10)
	| SMC_CYCLE_NRD_CYCLE(45));
	smc_set_mode(SMC, CONF_BOARD_HX8347A_LCD_CS, SMC_MODE_READ_MODE
	| SMC_MODE_WRITE_MODE | SMC_MODE_DBW_BIT_16);

	// Inicializova nastavenia obrazovky
	g_hx8347a_display_opt.ul_width = HX8347A_LCD_WIDTH;
	g_hx8347a_display_opt.ul_height = HX8347A_LCD_HEIGHT;
	g_hx8347a_display_opt.foreground_color = rgb24_to_rgb16(COLOR_WHITE);
	g_hx8347a_display_opt.background_color = rgb24_to_rgb16(COLOR_BLACK);

	// Vypn podsvietenie
	aat31xx_disable_backlight();

	// Inicializova obrazovku
	hx8347a_init(&g_hx8347a_display_opt);
	hx8347a_enable_scroll();

	// Nastavi podsvietenie
	aat31xx_set_backlight(AAT31XX_AVG_BACKLIGHT_LEVEL);
	__applcd_backlight = AAT31XX_AVG_BACKLIGHT_LEVEL;
	__applcd_backlight_on = true;

	// Vyisti displej
	hx8347a_set_foreground_color(rgb24_to_rgb16(COLOR_BLACK));
	hx8347a_draw_filled_rectangle(0, 0, HX8347A_LCD_WIDTH - 1, HX8347A_LCD_HEIGHT - 1);

	// Zapn displej
	hx8347a_display_on();
	__applcd_display_on = true;
}

// Vyist displej
void usbapp_lcd_clear(uint32_t color)
{
	__applcd_num_lines = 0;
	hx8347a_set_foreground_color(rgb24_to_rgb16(color));
	hx8347a_draw_filled_rectangle(0, 0, HX8347A_LCD_WIDTH - 1, HX8347A_LCD_HEIGHT - 1);
}

// Zape riadok na obrazovku
void usbapp_lcd_write_line_col(const char * line, uint32_t color)
{
	__applcd_num_lines++;
	if (__applcd_num_lines >= MAX_LINES)
	{
		__applcd_num_lines = 1;
		hx8347a_set_foreground_color(rgb24_to_rgb16(COLOR_BLACK));
		hx8347a_draw_filled_rectangle(0, 0, HX8347A_LCD_WIDTH - 1, HX8347A_LCD_HEIGHT - 1);
	}
	hx8347a_set_foreground_color(rgb24_to_rgb16(color));
	hx8347a_draw_string(2, 2 + (__applcd_num_lines - 1) * LINE_HEIGHT, (const uint8_t *)line);
}

// Zapne displej
void usbapp_lcd_on(void)
{
	hx8347a_display_on();
	__applcd_display_on = true;
	hx8347a_scroll(__applcd_scroll);
}

// Vypne displej
void usbapp_lcd_off(void)
{
	hx8347a_display_off();
	__applcd_display_on = false;
}

// Vypne podsvietenie
void usbapp_lcd_backlight_off(void)
{
	aat31xx_disable_backlight();
	__applcd_backlight_on = false;
}

// Nastav level podsvietenia
void usbapp_lcd_set_backlight(uint32_t level)
{
	if (level > 32)
		level = 32;
	aat31xx_disable_backlight();
	aat31xx_set_backlight(level);
	__applcd_backlight_on = level > 0;
	__applcd_backlight = (uint8_t)level;
}

// Vykresl bitmapu
void usbapp_lcd_draw_bitmap(uint32_t x, uint32_t y, uint32_t width, uint32_t height,
	uint16_t * bitmap)
{
	hx8347a_draw_pixmap(x, y, width, height, bitmap);
}

// Zska hodnotu urujcu, i je displej zapnut
bool usbapp_lcd_is_on(void)
{
	return __applcd_display_on;
}

// Zska hodnotu urujcu, i je podsvietenie zapnut
bool usbapp_lcd_is_backlight_on(void)
{
	return __applcd_backlight_on;
}

// Zska intenzitu podsvietenia
uint8_t usbapp_lcd_get_backlight(void)
{
	return __applcd_backlight;
}

// Skontroluje, i sa dan koordinty nachdzaj v priestore obrazovky
bool usbapp_lcd_check_coords(uint16_t posX, uint16_t posY, uint16_t width, uint16_t height)
{
	if (posX >= HX8347A_LCD_WIDTH || posY >= HX8347A_LCD_HEIGHT ||
		(posX + width) > HX8347A_LCD_WIDTH || (posY + height) > HX8347A_LCD_HEIGHT)
		return false;

	return true;
}

// Vykresl text na obrazovku
void usbapp_lcd_draw_text(const char * text, uint16_t posX, uint16_t posY, uint32_t color)
{
	hx8347a_set_foreground_color(rgb24_to_rgb16(color));
	hx8347a_draw_string(posX, posY, (const uint8_t *)text);
}

// Zska hodnotu skrolovania displeja
uint16_t usbapp_lcd_get_scroll(void)
{
	return __applcd_scroll;
}

// Nastav hodnotu skrolovania displeja
void usbapp_lcd_set_scroll(uint16_t scroll)
{
	__applcd_scroll = scroll >= HX8347A_LCD_HEIGHT ? 0 : scroll;
	hx8347a_scroll(__applcd_scroll);
}