#include "AppPWM.h"
#include "asf.h"

// PWM kanl pre LED didu
#define PWM_CHANNEL_LED PWM_CHANNEL_0
// PWM frekvencia v Hz
#define PWM_FREQUENCY 1000
// Perida vstupnej vlny PWM
#define PERIOD_VALUE 100
// Prvotn perida vstupnej vlny
#define INIT_DUTY_VALUE 0

// truktra nastavenia PWM kanla
static pwm_channel_t g_pwm_channel_led;
// Hodnota urujca, i je PWM kanl povolen
static volatile bool __apppwm_channel_enabled = false;
// Aktulna perida vstupnej vlny PWM
static volatile uint32_t __apppwm_duty = 0;

// Inicializuje pouvanie PWM
void usbapp_pwm_initialize(void)
{
	// Povoli PWM obvod
	pmc_enable_periph_clk(ID_PWM);
	
	// Vypn PWM kanly, aby sme ich mohli nakonfigurova
	pwm_channel_disable(PWM, PWM_CHANNEL_LED);
	
	// Inicializova PWM
	pwm_clock_t clock_setting = {
		.ul_clka = PWM_FREQUENCY * PERIOD_VALUE,
		.ul_clkb = 0,
		.ul_mck = sysclk_get_cpu_hz()
	};
	pwm_init(PWM, &clock_setting);
	
	// Inicializova PWM kanl pre LED didu
	g_pwm_channel_led.ul_prescaler = PWM_CMR_CPRE_CLKA;
	g_pwm_channel_led.ul_period = PERIOD_VALUE;
	g_pwm_channel_led.ul_duty = INIT_DUTY_VALUE;
	g_pwm_channel_led.channel = PWM_CHANNEL_LED;
	pwm_channel_init(PWM, &g_pwm_channel_led);
	__apppwm_duty = INIT_DUTY_VALUE;
}

// Povol PWM na LED dide
void usbapp_pwm_enable_led(void)
{
	pwm_channel_enable(PWM, PWM_CHANNEL_LED);
	__apppwm_channel_enabled = true;
}

// Vypne PWM na LED dide
void usbapp_pwm_disable_led(void)
{
	pwm_channel_disable(PWM, PWM_CHANNEL_LED);
	__apppwm_channel_enabled = false;
}

// Aktualizuje hodnotu "duty" (kedy je vlna aktvna)
void usbapp_pwm_set_led(uint32_t duty)
{
	if (__apppwm_channel_enabled)
	{
		if (duty > PERIOD_VALUE)
			duty = PERIOD_VALUE;
		pwm_channel_update_duty(PWM, &g_pwm_channel_led, duty);
		__apppwm_duty = duty;
	}		
}

// Zska hodnotu urujcu, i je PWM povolen
bool usbapp_pwm_is_enabled(void)
{
	return __apppwm_channel_enabled;
}

// Zska hodnotu "duty" (kedy je vlna aktvna)
uint32_t usbapp_pwm_get_led(void)
{
	return __apppwm_duty;
}