#include "AppQTouch.h"
#include "asf.h"

// Qtouch key number
#define BOARD_KEY_NUM_5
// The PIO numbers for silder is before key
#define BOARD_SILDER_BEFOR_KEY
// Qtouch valid key ID
#define BOARD_VALID_KEY_ID    1
// Qtouch up key ID
#define BOARD_UP_KEY_ID    2
// Qtouch down key ID
#define BOARD_DOWN_KEY_ID    3
// Qtouch left key ID
#define BOARD_LEFT_KEY_ID    4
// Qtouch right key ID
#define BOARD_RIGHT_KEY_ID    5
// Qtouch valid key channel
#define BOARD_VALID_KEY_CHANNEL    CHANNEL_25
// Qtouch up key channel
#define BOARD_UP_KEY_CHANNEL    CHANNEL_26
// Qtouch down key channel
#define BOARD_DOWN_KEY_CHANNEL    CHANNEL_27
// Qtouch left key channel
#define BOARD_LEFT_KEY_CHANNEL    CHANNEL_28
// Qtouch right key channel
#define BOARD_RIGHT_KEY_CHANNEL    CHANNEL_29
// Qtouch slider start channel
#define BOARD_SLIDER_START_CHANNEL    CHANNEL_0
// Qtouch slider end channel
#define BOARD_SLIDER_END_CHANNEL    CHANNEL_2

#define GET_SENSOR_STATE(SENSOR_NUMBER) (qt_measure_data.qt_touch_status.sensor_states[(SENSOR_NUMBER/8)] & (1 << (SENSOR_NUMBER % 8)))
#define GET_ROTOR_SLIDER_POSITION(ROTOR_SLIDER_NUMBER) qt_measure_data.qt_touch_status.rotor_slider_values[ROTOR_SLIDER_NUMBER]

static volatile uint16_t __appqtouch_status_flag = 0u;
static volatile uint16_t __appqtouch_burst_flag = 0u;
static volatile bool __appqtouch_qtouch_changed = false;

static volatile uint8_t __appqtouch_buttons = 0;
static volatile uint8_t __appqtouch_slider = 0;

// Skonfiguruje senzory
static void usbapp_qtouch_config_sensors(void)
{
	qt_enable_slider(BOARD_SLIDER_START_CHANNEL, BOARD_SLIDER_END_CHANNEL,
		AKS_GROUP_1, 16u, HYST_6_25, RES_8_BIT, 0u);
	
	qt_enable_key(BOARD_VALID_KEY_CHANNEL, AKS_GROUP_1, 18u, HYST_6_25);
	qt_enable_key(BOARD_UP_KEY_CHANNEL, AKS_GROUP_1, 18u, HYST_6_25);
	qt_enable_key(BOARD_DOWN_KEY_CHANNEL, AKS_GROUP_1, 18u, HYST_6_25);
	qt_enable_key(BOARD_LEFT_KEY_CHANNEL, AKS_GROUP_1, 18u, HYST_6_25);
	qt_enable_key(BOARD_RIGHT_KEY_CHANNEL, AKS_GROUP_1, 18u, HYST_6_25);
}

// Nastav parametre pre QTouch
static void usbapp_qtouch_set_parameters(void)
{
	qt_config_data.qt_di = DEF_QT_DI;
	qt_config_data.qt_neg_drift_rate = DEF_QT_NEG_DRIFT_RATE;
	qt_config_data.qt_pos_drift_rate = DEF_QT_POS_DRIFT_RATE;
	qt_config_data.qt_max_on_duration = DEF_QT_MAX_ON_DURATION;
	qt_config_data.qt_drift_hold_time = DEF_QT_DRIFT_HOLD_TIME;
	qt_config_data.qt_recal_threshold = DEF_QT_RECAL_THRESHOLD;
	qt_config_data.qt_pos_recal_delay = DEF_QT_POS_RECAL_DELAY;
}

// Inicializuje QTouch
void usbapp_qtouch_initialize(void)
{
	// Povoli obvody pre QTouch tlaidl
	pmc_enable_periph_clk(ID_PIOE);
	pmc_enable_periph_clk(ID_PIOF);
	
	// Resetova QTouch
	qt_reset_sensing();
	
	// Skonfigurova senzory
	usbapp_qtouch_config_sensors();
	
	// Inicializuje snmanie dotykov tlaidiel
	qt_init_sensing();
	
	usbapp_qtouch_set_parameters();
	qt_filter_callback = 0;
}

// Zska hodnotu urujcu, i sa od poslednho merania zmenil stav tlaidiel
bool usbapp_qtouch_changed(void)
{
	return __appqtouch_qtouch_changed;
}

// Odstrni prznak toho, i sa od poslednho meranie zmenil stav tlaidiel
void usbapp_qtouch_clear_changed(void)
{
	__appqtouch_qtouch_changed = false;
}

// Zachyt aktulny stav QTouch
void usbapp_qtouch_measure(uint32_t timeMs)
{
	do
	{
		__appqtouch_status_flag = qt_measure_sensors(timeMs);
		__appqtouch_burst_flag = __appqtouch_status_flag & QTLIB_BURST_AGAIN;
	} while (__appqtouch_burst_flag);
	
	if ((GET_SENSOR_STATE(BOARD_VALID_KEY_ID) != 0) && !(__appqtouch_buttons & QTouchButton_Valid))
	{
		__appqtouch_buttons |= QTouchButton_Valid;
		__appqtouch_qtouch_changed = true;
	}
	else
	{
		if ((GET_SENSOR_STATE(BOARD_VALID_KEY_ID) == 0)
			&& (__appqtouch_buttons & QTouchButton_Valid))
		{
			__appqtouch_buttons &= (~QTouchButton_Valid);
			__appqtouch_qtouch_changed = true;
		}
	}
	if ((GET_SENSOR_STATE(BOARD_UP_KEY_ID) != 0) && !(__appqtouch_buttons & QTouchButton_Up))
	{
		__appqtouch_buttons |= QTouchButton_Up;
		__appqtouch_qtouch_changed = true;
	}
	else
	{
		if ((GET_SENSOR_STATE(BOARD_UP_KEY_ID) == 0)
		&& (__appqtouch_buttons & QTouchButton_Up))
		{
			__appqtouch_buttons &= (~QTouchButton_Up);
			__appqtouch_qtouch_changed = true;
		}
	}
	if ((GET_SENSOR_STATE(BOARD_RIGHT_KEY_ID) != 0) && !(__appqtouch_buttons & QTouchButton_Right))
	{
		__appqtouch_buttons |= QTouchButton_Right;
		__appqtouch_qtouch_changed = true;
	}
	else
	{
		if ((GET_SENSOR_STATE(BOARD_RIGHT_KEY_ID) == 0) && (__appqtouch_buttons & QTouchButton_Right))
		{
			__appqtouch_buttons &= (~QTouchButton_Right);
			__appqtouch_qtouch_changed = true;
		}
	}
	if ((GET_SENSOR_STATE(BOARD_DOWN_KEY_ID) != 0) && !(__appqtouch_buttons & QTouchButton_Down))
	{
		__appqtouch_buttons |= QTouchButton_Down;
		__appqtouch_qtouch_changed = true;
	}
	else
	{
		if ((GET_SENSOR_STATE(BOARD_DOWN_KEY_ID) == 0) && (__appqtouch_buttons & QTouchButton_Down))
		{
			__appqtouch_buttons &= (~QTouchButton_Down);
			__appqtouch_qtouch_changed = true;
		}
	}
	if ((GET_SENSOR_STATE(BOARD_LEFT_KEY_ID) != 0) && !(__appqtouch_buttons & QTouchButton_Left))
	{
		__appqtouch_buttons |= QTouchButton_Left;
		__appqtouch_qtouch_changed = true;
	}
	else
	{
		if ((GET_SENSOR_STATE(BOARD_LEFT_KEY_ID) == 0) && (__appqtouch_buttons & QTouchButton_Left))
		{
			__appqtouch_buttons &= (~QTouchButton_Left);
			__appqtouch_qtouch_changed = true;
		}
	}
	
	if (GET_ROTOR_SLIDER_POSITION(0) != __appqtouch_slider)
	{
		__appqtouch_slider = GET_ROTOR_SLIDER_POSITION(0);
		__appqtouch_qtouch_changed = true;
	}
}

// Zska hodnotu, ktor tlaidl s stlaen
uint8_t usbapp_qtouch_buttons(void)
{
	return __appqtouch_buttons;
}

// Zska hodnotu slideru
uint8_t usbapp_qtouch_slider(void)
{
	return __appqtouch_slider;
}